/**
 * @file tb_prng.cpp
 * @brief Checks that shake_prng generates the same stream of bytes as the reference code when initialised identically.
 */

#include "shake_prng.h"
#include <stdio.h>
#include <stdint.h>
#include <assert.h>

int tb_generic_call(void (*f)(uint8_t*, size_t), const char *name, size_t compare_bytesize);

/**
 * @brief Initializes prng, calls shake_prng and writes the generated bytes in hex into output.
 *
 * Initialises the PRNG with 0x43 as initial entropy byte, calls shake_prng and
 * writes compare_bytesize generated bytes in hex into output.
 *
 * @param[out] output pointer to a byte array containing the generated bytes in hex
 * @param[in] compare_bytesize amount of bytes that are generated and will be compared with the reference data. Must be a multiple of 8.
 */
void tested_function(uint8_t *output, size_t compare_bytesize) {
    //prng
    ap_uint8 entropy = 0x43;
    ap_uint64 prng_state[26];

    shake_prng_init(&entropy, NULL, 1, 0, prng_state);
    assert(compare_bytesize % 8 == 0);
    shake_prng((ap_uint64 *)output, compare_bytesize / 8, prng_state);
}


/**
 * @brief Launches the tb_generic_call function that calls tested_function and checks the resulting output is correct.
 *
 * Calls the generic function that will call tested_function to generate the hardware implementation output,
 * and then compare this output to the reference data which is available on a pre-generated file.
 *
 * @returns 0 if the outputs are the same and 1 if not
 */
int main() {
    const char *test_name = "prng";
    return tb_generic_call(tested_function, test_name, 16);
}
